@@ -50,7 +50,6 @@
   #include <regex.h>
 #endif
 
-
 /* ppriv - NO_INTERACTIVE_DEPENDENCY - for simpler debugging in Visual Studio
  *
  */
@@ -60,17 +59,19 @@
 #endif
 
 #include "util/omc_error.h"
+#include "util/omc_file.h"
+#include "util/omc_numbers.h"
 #include "simulation_data.h"
 #include "openmodelica_func.h"
 #include "meta/meta_modelica.h"
 
-#include "linearize.h"
+#include "linearization/linearize.h"
 #include "options.h"
 #include "simulation_runtime.h"
 #include "simulation_input_xml.h"
 #include "simulation/results/simulation_result_plt.h"
 #include "simulation/results/simulation_result_csv.h"
-#include "simulation/results/simulation_result_mat.h"
+#include "simulation/results/simulation_result_mat4.h"
 #include "simulation/results/simulation_result_wall.h"
 #include "simulation/results/simulation_result_ia.h"
 #include "simulation/solver/solver_main.h"
@@ -84,6 +85,9 @@
 #include "util/rtclock.h"
 #include "omc_config.h"
 #include "simulation/solver/initialization/initialization.h"
+#include "simulation/solver/dae_mode.h"
+#include "dataReconciliation/dataReconciliation.h"
+#include "util/parallel_helper.h"
 
 #ifdef _OMC_QSS_LIB
   #include "solver_qss/solver_qss.h"
@@ -104,7 +108,7 @@ int sim_noemit = 0;           /* Flag for not emitting data */
 const std::string *init_method = NULL; /* method for  initialization. */
 
 static int callSolver(DATA* simData, threadData_t *threadData, string init_initMethod, string init_file,
-      double init_time, int lambda_steps, string outputVariablesAtEnd, int cpuTime, const char *argv_0);
+      double init_time, string outputVariablesAtEnd, int cpuTime, const char *argv_0);
 
 /*! \fn void setGlobalVerboseLevel(int argc, char**argv)
  *
@@ -126,9 +130,14 @@ void setGlobalVerboseLevel(int argc, char**argv)
     /* default activated */
     useStream[LOG_STDOUT] = 1;
     useStream[LOG_ASSERT] = 1;
+    useStream[LOG_SUCCESS] = 1;
     return; // no lv flag given.
   }
 
+  /* default activated, but it can be disabled with -stdout or -assert */
+  useStream[LOG_STDOUT] = 1;
+  useStream[LOG_ASSERT] = 1;
+
   if(flags->find("LOG_ALL", 0) != string::npos)
   {
     for(i=1; i<SIM_LOG_MAX; ++i)
@@ -161,6 +170,11 @@ void setGlobalVerboseLevel(int argc, char**argv)
           useStream[i] = 1;
           error = 0;
         }
+        else if(flag == string("-") + string(LOG_STREAM_NAME[i]))
+        {
+          useStream[i] = 0;
+          error = 0;
+        }
       }
 
       if(error)
@@ -174,13 +188,20 @@ void setGlobalVerboseLevel(int argc, char**argv)
     }while(pos != string::npos);
   }
 
-  /* default activated */
-  useStream[LOG_STDOUT] = 1;
-  useStream[LOG_ASSERT] = 1;
-
-  /* print LOG_SOTI if LOG_INIT is enabled */
-  if(useStream[LOG_INIT])
+  /* print LOG_INIT and LOG_SOTI if LOG_INIT_V is active */
+  if(useStream[LOG_INIT_V] == 1)
+  {
+    useStream[LOG_INIT] = 1;
     useStream[LOG_SOTI] = 1;
+  }
+
+  /* print LOG_INIT_HOMOTOPY if LOG_INIT is active */
+  if(useStream[LOG_INIT] == 1)
+    useStream[LOG_INIT_HOMOTOPY] = 1;
+
+  /* print LOG_STATS if LOG_SOLVER if active */
+  if(useStream[LOG_SOLVER_V] == 1)
+    useStream[LOG_SOLVER] = 1;
 
   /* print LOG_STATS if LOG_SOLVER if active */
   if(useStream[LOG_SOLVER] == 1)
@@ -214,114 +235,71 @@ void setGlobalVerboseLevel(int argc, char**argv)
   delete flags;
 }
 
-static int getNonlinearSolverMethod()
+
+/* Read value of flag lv_time to set time interval in which loggin is active */
+void setGlobalLoggingTime(int argc, char**argv, SIMULATION_INFO *simulationInfo)
 {
-  int i;
-  const char *cflags = omc_flagValue[FLAG_NLS];
-  const string *method = cflags ? new string(cflags) : NULL;
+  const char *flagStr = omc_flagValue[FLAG_LV_TIME];
+  const string *flags = flagStr ? new string(flagStr) : NULL;
+  char *endptr;
+  const char *secondPart;
+  double loggingStartTime, loggingStopTime;
 
-  if(!method)
-    return NLS_MIXED; /* default method */
+  /* Check if lv_time flag is given */
+  if (flagStr==NULL || *flagStr=='\0')
+  {
+    /* default activated --> Log everything*/
+    simulationInfo->useLoggingTime = 0;
+    return;
+  }
 
-  for(i=1; i<NLS_MAX; ++i)
-    if(*method == NLS_NAME[i])
-      return i;
+  /* Parse flagStr */
+  loggingStartTime = om_strtod(flagStr, &endptr);
+  endptr = endptr+1;
+  secondPart = endptr;
+  loggingStopTime = om_strtod(secondPart, &endptr);
+  if (*endptr)
+  {
+    throwStreamPrint(NULL, "Simulation flag %s expects two real numbers, seperated by a comata. Got: %s", FLAG_NAME[FLAG_LV_TIME], flagStr);
+  }
 
-  warningStreamPrint(LOG_STDOUT, 1, "unrecognized option -nls=%s, current options are:", method->c_str());
-  for(i=1; i<NLS_MAX; ++i)
-    warningStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", NLS_NAME[i], NLS_DESC[i]);
-  messageClose(LOG_STDOUT);
-  throwStreamPrint(NULL,"see last warning");
+  /* Check flag input */
+  if (loggingStartTime > loggingStopTime)
+  {
+    throwStreamPrint(NULL, "Simulation flag %s expects first number to be smaller then second number. Got: %s", FLAG_NAME[FLAG_LV_TIME], flagStr);
+  }
 
-  return NLS_NONE;
+  /* Save logging time */
+  simulationInfo->useLoggingTime = 1;
+  simulationInfo->loggingTimeRecord[0] = loggingStartTime;
+  simulationInfo->loggingTimeRecord[1] = loggingStopTime;
+  infoStreamPrint(LOG_STDOUT, 0, "Time dependent logging enabled. Activate loggin in intervall [%f, %f]", simulationInfo->loggingTimeRecord[0], simulationInfo->loggingTimeRecord[1]);
+
+  /* Deactivate Logging */
+  deactivateLogging();
 }
 
-static int getlinearSolverMethod()
+
+static void readFlag(int *flag, int max, const char *value, const char *flagName, const char **names, const char **desc)
 {
   int i;
-  const char *cflags = omc_flagValue[FLAG_LS];
-  const string *method = cflags ? new string(cflags) : NULL;
+  if (!value) {
+    return; /* keep the default value */
+  }
 
-  if(!method)
-    return LS_DEFAULT; /* default method */
+  for (i=1; i<max; ++i) {
+    if (0 == strcmp(value, names[i])) {
+      *flag = i;
+      return;
+    }
+  }
 
-  for(i=1; i<LS_MAX; ++i)
-    if(*method == LS_NAME[i])
-      return i;
-
-  warningStreamPrint(LOG_STDOUT, 1, "unrecognized option -ls=%s, current options are:", method->c_str());
-  for(i=1; i<LS_MAX; ++i)
-    warningStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", LS_NAME[i], LS_DESC[i]);
+  warningStreamPrint(LOG_STDOUT, 1, "unrecognized option %s=%s, current options are:", flagName, value);
+  for (i=1; i<max; ++i) {
+    warningStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", names[i], desc[i]);
+  }
   messageClose(LOG_STDOUT);
   throwStreamPrint(NULL,"see last warning");
-
-  return LS_NONE;
-}
-
-static int getlinearSparseSolverMethod()
-{
-  int i;
-  const char *cflags = omc_flagValue[FLAG_LSS];
-  const string *method = cflags ? new string(cflags) : NULL;
-
-  if(!method)
-    return LSS_KLU; /* default method */
-
-  for(i=1; i<LSS_MAX; ++i)
-    if(*method == LSS_NAME[i])
-      return i;
-
-  warningStreamPrint(LOG_STDOUT, 1, "unrecognized option -lss=%s, current options are:", method->c_str());
-  for(i=1; i<LSS_MAX; ++i)
-    warningStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", LSS_NAME[i], LSS_DESC[i]);
-  messageClose(LOG_STDOUT);
-  throwStreamPrint(NULL,"see last warning");
-
-  return LSS_NONE;
-}
-
-static int getNewtonStrategy()
-{
-  int i;
-  const char *cflags = omc_flagValue[FLAG_NEWTON_STRATEGY];
-  const string *method = cflags ? new string(cflags) : NULL;
-
-  if(!method)
-    return NEWTON_DAMPED2; /* default method */
-
-  for(i=1; i<NEWTON_MAX; ++i)
-    if(*method == NEWTONSTRATEGY_NAME[i])
-      return i;
-
-  warningStreamPrint(LOG_STDOUT, 1, "unrecognized option -nls=%s, current options are:", method->c_str());
-  for(i=1; i<NEWTON_MAX; ++i)
-    warningStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", NEWTONSTRATEGY_NAME[i], NEWTONSTRATEGY_DESC[i]);
-  messageClose(LOG_STDOUT);
-  throwStreamPrint(NULL,"see last warning");
-
-  return NEWTON_NONE;
-}
-
-static int getNlsLSSolver()
-{
-  int i;
-  const char *cflags = omc_flagValue[FLAG_NLS_LS];
-  const string *method = cflags ? new string(cflags) : NULL;
-
-  if(!method)
-    return NLS_LS_LAPACK; /* default method */
-
-  for(i=1; i<NLS_LS_MAX; ++i)
-    if(*method == NLS_LS_METHOD[i])
-      return i;
-
-  warningStreamPrint(LOG_STDOUT, 1, "unrecognized option -nls=%s, current options are:", method->c_str());
-  for(i=1; i<NLS_LS_MAX; ++i)
-    warningStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", NLS_LS_METHOD[i], NLS_LS_METHOD_DESC[i]);
-  messageClose(LOG_STDOUT);
-  throwStreamPrint(NULL,"see last warning");
-
-  return NLS_LS_UNKNOWN;
 }
 
 static double getFlagReal(enum _FLAG flag, double res)
@@ -331,7 +309,7 @@ static double getFlagReal(enum _FLAG flag, double res)
   if (flagStr==NULL || *flagStr=='\0') {
     return res;
   }
-  res = strtod(flagStr, &endptr);
+  res = om_strtod(flagStr, &endptr);
   if (*endptr) {
     throwStreamPrint(NULL, "Simulation flag %s expects a real number, got: %s", FLAG_NAME[flag], flagStr);
   }
@@ -493,6 +471,7 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
     rt_measure_overhead(SIM_TIMER_TOTAL);
     rt_clear(SIM_TIMER_TOTAL);
     rt_tick(SIM_TIMER_TOTAL);
+    rt_clear(SIM_TIMER_PREINIT);
     rt_tick(SIM_TIMER_PREINIT);
     rt_clear(SIM_TIMER_OUTPUT);
     rt_clear(SIM_TIMER_EVENT);
@@ -506,7 +485,7 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
     } else {
       data->simulationInfo->stopTime = atof(lintime);
     }
-    infoStreamPrint(LOG_STDOUT, 0, "Linearization will performed at point of time: %f", data->simulationInfo->stopTime);
+    infoStreamPrint(LOG_STDOUT, 0, "Linearization will be performed at point of time: %f", data->simulationInfo->stopTime);
   }
 
   /* set delta x for linearization */
@@ -531,23 +510,31 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
       infoStreamPrint(LOG_SOLVER, 0, "overwrite solver method: %s [from command line]", data->simulationInfo->solverMethod);
     }
   }
+  /* if the model is compiled in daeMode then we have to use ida solver */
+  if (compiledInDAEMode && std::string("ida") != data->simulationInfo->solverMethod) {
+    data->simulationInfo->solverMethod = GC_strdup(std::string("ida").c_str());
+    infoStreamPrint(LOG_SIMULATION, 0, "overwrite solver method: %s [DAEmode works only with IDA solver]", data->simulationInfo->solverMethod);
+  }
 
   // Create a result file
   const char *result_file = omc_flagValue[FLAG_R];
   string result_file_cstr;
-  if(!result_file) {
+  if (result_file) {
+    data->modelData->resultFileName = GC_strdup(result_file);
+  } else if (omc_flag[FLAG_OUTPUT_PATH]) { /* read the output path from the command line (if any) */
+    if (0 > GC_asprintf(&result_file, "%s/%s_res.%s", omc_flagValue[FLAG_OUTPUT_PATH], data->modelData->modelFilePrefix, data->simulationInfo->outputFormat)) {
+      throwStreamPrint(NULL, "simulation_runtime.c: Error: can not allocate memory.");
+    }
+    data->modelData->resultFileName = GC_strdup(result_file);
+  } else {
     result_file_cstr = string(data->modelData->modelFilePrefix) + string("_res.") + data->simulationInfo->outputFormat;
     data->modelData->resultFileName = GC_strdup(result_file_cstr.c_str());
-  } else {
-    data->modelData->resultFileName = GC_strdup(result_file);
   }
 
   string init_initMethod = "";
   string init_file = "";
   string init_time_string = "";
   double init_time = 0.0;
-  string init_lambda_steps_string = "";
-  int init_lambda_steps = 1;
   string outputVariablesAtEnd = "";
   int cpuTime = omc_flag[FLAG_CPU];
 
@@ -555,15 +542,30 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
     init_initMethod = omc_flagValue[FLAG_IIM];
   }
   if(omc_flag[FLAG_IIF]) {
-    init_file = omc_flagValue[FLAG_IIF];
+    if (omc_flag[FLAG_INPUT_PATH]) {
+      const char *tmp_filename;
+      if (0 > GC_asprintf(&tmp_filename, "%s/%s", omc_flagValue[FLAG_INPUT_PATH], omc_flagValue[FLAG_IIF])) {
+        throwStreamPrint(NULL, "simulation_runtime.cpp: Error: can not allocate memory.");
+      }
+      init_file = tmp_filename;
+    }
+    else {
+      init_file = omc_flagValue[FLAG_IIF];
+    }
   }
   if(omc_flag[FLAG_IIT]) {
     init_time_string = omc_flagValue[FLAG_IIT];
     init_time = atof(init_time_string.c_str());
   }
   if(omc_flag[FLAG_ILS]) {
-    init_lambda_steps_string = omc_flagValue[FLAG_ILS];
-    init_lambda_steps = atoi(init_lambda_steps_string.c_str());
+    init_lambda_steps = atoi(omc_flagValue[FLAG_ILS]);
+    if(init_lambda_steps <= 0) {
+      init_lambda_steps = 0;
+      infoStreamPrint(LOG_STDOUT, 0, "Number of lambda steps set to 0. Homotopy is disabled.");
+    }
+    else {
+      infoStreamPrint(LOG_STDOUT, 0, "Number of lambda steps for homotopy approach changed to %d", init_lambda_steps);
+    }
   }
   if(omc_flag[FLAG_MAX_BISECTION_ITERATIONS]) {
     maxBisectionIterations = atoi(omc_flagValue[FLAG_MAX_BISECTION_ITERATIONS]);
@@ -577,17 +579,34 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
     outputVariablesAtEnd = omc_flagValue[FLAG_OUTPUT];
   }
 
-  retVal = callSolver(data, threadData, init_initMethod, init_file, init_time, init_lambda_steps, outputVariablesAtEnd, cpuTime, argv[0]);
+  /* Check if logging should be enabled */
+  if ((data->simulationInfo->useLoggingTime == 1) && (data->simulationInfo->startTime >= data->simulationInfo->loggingTimeRecord[0])) {
+    reactivateLogging();
+  }
+
+  retVal = callSolver(data, threadData, init_initMethod, init_file, init_time, outputVariablesAtEnd, cpuTime, argv[0]);
+
+  /* Check if logging should be disabled */
+  if (data->simulationInfo->useLoggingTime == 1) {
+    deactivateLogging();
+  }
 
   if (omc_flag[FLAG_ALARM]) {
     alarm(0);
   }
 
+  if(0 == retVal && omc_flag[FLAG_DATA_RECONCILE])
+  {
+  infoStreamPrint(LOG_STDOUT, 0, "DataReconciliation Starting!");
+  infoStreamPrint(LOG_STDOUT, 0, "%s", data->modelData->modelName);
+  retVal = dataReconciliation(data, threadData);
+  infoStreamPrint(LOG_STDOUT, 0, "DataReconciliation Completed!");
+  }
+
   if(0 == retVal && create_linearmodel) {
-    rt_tick(SIM_TIMER_LINEARIZE);
+    rt_tick(SIM_TIMER_JACOBIAN);
     retVal = linearize(data, threadData);
-    rt_accumulate(SIM_TIMER_LINEARIZE);
-    infoStreamPrint(LOG_STDOUT, 0, "Linear model is created!");
+    rt_accumulate(SIM_TIMER_JACOBIAN);
   }
 
   /* Use the saved state of measure_time_flag.
@@ -595,16 +614,19 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
    * So before doing the profiling reset the measure_time_flag to measure_time_flag_previous state.
    */
   measure_time_flag = measure_time_flag_previous;
-
-  if(0 == retVal && measure_time_flag) {
+  string output_path = "";
+  if (0 == retVal && measure_time_flag) {
+    if (omc_flag[FLAG_OUTPUT_PATH]) { /* read the output path from the command line (if any) */
+      output_path = string(omc_flagValue[FLAG_INPUT_PATH]) + string("/");
+    }
     const string jsonInfo = string(data->modelData->modelFilePrefix) + "_prof.json";
     const string modelInfo = string(data->modelData->modelFilePrefix) + "_prof.xml";
     const string plotFile = string(data->modelData->modelFilePrefix) + "_prof.plt";
     rt_accumulate(SIM_TIMER_TOTAL);
     const char* plotFormat = omc_flagValue[FLAG_MEASURETIMEPLOTFORMAT];
-    retVal = printModelInfo(data, threadData, modelInfo.c_str(), plotFile.c_str(), plotFormat ? plotFormat : "svg",
+    retVal = printModelInfo(data, threadData, output_path.c_str(), modelInfo.c_str(), plotFile.c_str(), plotFormat ? plotFormat : "svg",
         data->simulationInfo->solverMethod, data->simulationInfo->outputFormat, data->modelData->resultFileName) && retVal;
-    retVal = printModelInfoJSON(data, threadData, jsonInfo.c_str(), data->modelData->resultFileName) && retVal;
+    retVal = printModelInfoJSON(data, threadData, output_path.c_str(), jsonInfo.c_str(), data->modelData->resultFileName) && retVal;
   }
 
   TRACE_POP
@@ -634,10 +656,10 @@ int initializeResultData(DATA* simData, threadData_t *threadData, int cpuTime)
     /* sim_result.writeParameterData = omc_csv_writeParameterData; */
     sim_result.free = omc_csv_free;
   } else if(0 == strcmp("mat", simData->simulationInfo->outputFormat)) {
-    sim_result.init = mat4_init;
-    sim_result.emit = mat4_emit;
-    sim_result.writeParameterData = mat4_writeParameterData;
-    sim_result.free = mat4_free;
+    sim_result.init = mat4_init4;
+    sim_result.emit = mat4_emit4;
+    sim_result.writeParameterData = mat4_writeParameterData4;
+    sim_result.free = mat4_free4;
     resultFormatHasCheapAliasesAndParameters = 1;
 #if !defined(OMC_MINIMAL_RUNTIME)
   } else if(0 == strcmp("wall", simData->simulationInfo->outputFormat)) {
@@ -678,7 +700,7 @@ int initializeResultData(DATA* simData, threadData_t *threadData, int cpuTime)
  * "rungekutta" calls a fourth-order Runge-Kutta Solver
  */
 static int callSolver(DATA* simData, threadData_t *threadData, string init_initMethod, string init_file,
-      double init_time, int lambda_steps, string outputVariablesAtEnd, int cpuTime, const char *argv_0)
+      double init_time, string outputVariablesAtEnd, int cpuTime, const char *argv_0)
 {
   TRACE_PUSH
   int retVal = -1;
@@ -710,8 +732,20 @@ static int callSolver(DATA* simData, threadData_t *threadData, string init_initM
   /* if no states are present, then we can
    * use euler method, since it does nothing.
    */
-  if (simData->modelData->nStates < 1 && solverID != S_OPTIMIZATION && solverID != S_SYM_EULER) {
+  if ( (simData->modelData->nStates < 1 &&
+        solverID != S_OPTIMIZATION &&
+        solverID != S_SYM_SOLVER &&
+        !compiledInDAEMode) ||
+       (compiledInDAEMode && (simData->simulationInfo->daeModeData->nResidualVars +
+                             simData->simulationInfo->daeModeData->nAlgebraicDAEVars < 1))
+      )
+  {
     solverID = S_EULER;
+    infoStreamPrint(LOG_SOLVER, 0, "No states present, continuing without ODE solver.");
+    if (compiledInDAEMode)
+    {
+      simData->callback->functionDAE = evaluateDAEResiduals_wrapperEventUpdate;
+    }
   }
 
   if(S_UNKNOWN == solverID) {
@@ -732,7 +766,7 @@ static int callSolver(DATA* simData, threadData_t *threadData, string init_initM
                         simData->simulationInfo->numSteps, simData->simulationInfo->tolerance, 3);
     } else /* standard solver interface */
 #endif
-      retVal = solver_main(simData, threadData, init_initMethod.c_str(), init_file.c_str(), init_time, lambda_steps, solverID, outVars, argv_0);
+      retVal = solver_main(simData, threadData, init_initMethod.c_str(), init_file.c_str(), init_time, solverID, outVars, argv_0);
   }
 
   MMC_CATCH_INTERNAL(mmc_jumper)
@@ -753,7 +787,29 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
   int i;
   initDumpSystem();
 
-  if(setLogFormat(argc, argv) || helpFlagSet(argc, argv) || checkCommandLineArguments(argc, argv))
+  int checkArgumentsRes = checkCommandLineArguments(argc, argv);
+
+#ifndef NO_INTERACTIVE_DEPENDENCY
+  if(omc_flag[FLAG_PORT]) {
+    std::istringstream stream(omc_flagValue[FLAG_PORT]);
+    int port;
+    stream >> port;
+    sim_communication_port_open = 1;
+    sim_communication_port_open &= sim_communication_port.create();
+    sim_communication_port_open &= sim_communication_port.connect("127.0.0.1", port);
+  }
+#endif
+
+  int logFormatResult = setLogFormat(argc, argv);
+
+#ifndef NO_INTERACTIVE_DEPENDENCY
+  if (isXMLTCP && !sim_communication_port_open) {
+    errorStreamPrint(LOG_STDOUT, 0, "xmltcp log format requires a TCP-port to be passed (and successfully open)");
+    EXIT(1);
+  }
+#endif
+
+  if(logFormatResult || helpFlagSet(argc, argv) || checkArgumentsRes)
   {
     infoStreamPrint(LOG_STDOUT, 1, "usage: %s", argv[0]);
 
@@ -791,21 +847,65 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
         /* detailed information for some flags */
         switch(i)
         {
-        case FLAG_LV:
-          for(j=firstOMCErrorStream; j<SIM_LOG_MAX; ++j)
-            infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", LOG_STREAM_NAME[j], LOG_STREAM_DESC[j]);
-          break;
+          case FLAG_IDA_LS:
+            for(j=1; j<IDA_LS_MAX; ++j) {
+              infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", IDA_LS_METHOD[j], IDA_LS_METHOD_DESC[j]);
+            }
+            break;
 
-        case FLAG_IIM:
-          for(j=1; j<IIM_MAX; ++j)
-            infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", INIT_METHOD_NAME[j], INIT_METHOD_DESC[j]);
-          break;
+          case FLAG_IIM:
+            for(j=1; j<IIM_MAX; ++j) {
+              infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", INIT_METHOD_NAME[j], INIT_METHOD_DESC[j]);
+            }
+            break;
 
-        case FLAG_S:
-          for(j=1; j<S_MAX; ++j) {
-            infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", SOLVER_METHOD_NAME[j], SOLVER_METHOD_DESC[j]);
-          }
-          break;
+          case FLAG_JACOBIAN:
+            for(j=1; j<JAC_MAX; ++j) {
+              infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", JACOBIAN_METHOD[j], JACOBIAN_METHOD_DESC[j]);
+            }
+            break;
+
+          case FLAG_LS:
+            for(j=1; j<LS_MAX; ++j) {
+              infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", LS_NAME[j], LS_DESC[j]);
+            }
+            break;
+
+          case FLAG_LSS:
+            for(j=1; j<LSS_MAX; ++j) {
+              infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", LSS_NAME[j], LSS_DESC[j]);
+            }
+            break;
+
+          case FLAG_LV:
+            for(j=firstOMCErrorStream; j<SIM_LOG_MAX; ++j) {
+              infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", LOG_STREAM_NAME[j], LOG_STREAM_DESC[j]);
+            }
+            break;
+
+          case FLAG_NEWTON_STRATEGY:
+            for(j=firstOMCErrorStream; j<NEWTON_MAX; ++j) {
+              infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", NEWTONSTRATEGY_NAME[j], NEWTONSTRATEGY_DESC[j]);
+            }
+            break;
+
+          case FLAG_NLS:
+            for(j=firstOMCErrorStream; j<NLS_MAX; ++j) {
+              infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", NLS_NAME[j], NLS_DESC[j]);
+            }
+            break;
+
+          case FLAG_NLS_LS:
+            for(j=firstOMCErrorStream; j<NLS_LS_MAX; ++j) {
+              infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", NLS_LS_METHOD[j], NLS_LS_METHOD_DESC[j]);
+            }
+            break;
+
+          case FLAG_S:
+            for(j=1; j<S_MAX; ++j) {
+              infoStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", SOLVER_METHOD_NAME[j], SOLVER_METHOD_DESC[j]);
+            }
+            break;
         }
         messageClose(LOG_STDOUT);
 
@@ -819,6 +919,7 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
   }
 
   setGlobalVerboseLevel(argc, argv);
+  setGlobalLoggingTime(argc, argv, data->simulationInfo);
   initializeDataStruc(data, threadData);
   if(!data)
   {
@@ -826,12 +927,73 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
     EXIT(1);
   }
 
-  data->simulationInfo->nlsMethod = getNonlinearSolverMethod();
-  data->simulationInfo->lsMethod = getlinearSolverMethod();
-  data->simulationInfo->lssMethod = getlinearSparseSolverMethod();
-  data->simulationInfo->newtonStrategy = getNewtonStrategy();
+  readFlag(&data->simulationInfo->nlsMethod, NLS_MAX, omc_flagValue[FLAG_NLS], "-nls", NLS_NAME, NLS_DESC);
+  readFlag(&data->simulationInfo-> lsMethod,  LS_MAX, omc_flagValue[FLAG_LS ],  "-ls",  LS_NAME,  LS_DESC);
+  readFlag(&data->simulationInfo->lssMethod, LSS_MAX, omc_flagValue[FLAG_LSS], "-lss", LSS_NAME, LSS_DESC);
+  readFlag(&homBacktraceStrategy, HOM_BACK_STRAT_MAX, omc_flagValue[FLAG_HOMOTOPY_BACKTRACE_STRATEGY], "-homBacktraceStrategy", HOM_BACK_STRAT_NAME, HOM_BACK_STRAT_DESC);
+  readFlag(&data->simulationInfo->newtonStrategy, NEWTON_MAX, omc_flagValue[FLAG_NEWTON_STRATEGY], "-newton", NEWTONSTRATEGY_NAME, NEWTONSTRATEGY_DESC);
   data->simulationInfo->nlsCsvInfomation = omc_flag[FLAG_NLS_INFO];
-  data->simulationInfo->nlsLinearSolver = getNlsLSSolver();
+  readFlag(&data->simulationInfo->nlsLinearSolver, NLS_LS_MAX, omc_flagValue[FLAG_NLS_LS], "-nlsLS", NLS_LS_METHOD, NLS_LS_METHOD_DESC);
+
+  if(omc_flag[FLAG_HOMOTOPY_ADAPT_BEND]) {
+    homAdaptBend = atof(omc_flagValue[FLAG_HOMOTOPY_ADAPT_BEND]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homAdaptBend changed to %f", homAdaptBend);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_H_EPS]) {
+    homHEps = atof(omc_flagValue[FLAG_HOMOTOPY_H_EPS]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homHEps changed to %f", homHEps);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_MAX_LAMBDA_STEPS]) {
+    homMaxLambdaSteps = atoi(omc_flagValue[FLAG_HOMOTOPY_MAX_LAMBDA_STEPS]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homMaxLambdaSteps changed to %d", homMaxLambdaSteps);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_MAX_NEWTON_STEPS]) {
+    homMaxNewtonSteps = atoi(omc_flagValue[FLAG_HOMOTOPY_MAX_NEWTON_STEPS]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homMaxNewtonSteps changed to %d", homMaxNewtonSteps);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_MAX_TRIES]) {
+    homMaxTries = atoi(omc_flagValue[FLAG_HOMOTOPY_MAX_TRIES]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homMaxTries changed to %d", homMaxTries);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_DEC_FACTOR]) {
+    homTauDecreasingFactor = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_DEC_FACTOR]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauDecreasingFactor changed to %f", homTauDecreasingFactor);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_DEC_FACTOR_PRED]) {
+    homTauDecreasingFactorPredictor = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_DEC_FACTOR_PRED]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauDecreasingFactorPredictor changed to %f", homTauDecreasingFactorPredictor);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_INC_FACTOR]) {
+    homTauIncreasingFactor = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_INC_FACTOR]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauIncreasingFactor changed to %f", homTauIncreasingFactor);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_INC_THRESHOLD]) {
+    homTauIncreasingThreshold = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_INC_THRESHOLD]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauIncreasingThreshold changed to %f", homTauIncreasingThreshold);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_MAX]) {
+    homTauMax = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_MAX]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauMax changed to %f", homTauMax);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_MIN]) {
+    homTauMin = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_MIN]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauMin changed to %f", homTauMin);
+  }
+
+  if(omc_flag[FLAG_HOMOTOPY_TAU_START]) {
+    homTauStart = atof(omc_flagValue[FLAG_HOMOTOPY_TAU_START]);
+    infoStreamPrint(LOG_STDOUT, 0, "homotopy parameter homTauStart changed to %f", homTauStart);
+  }
 
   if(omc_flag[FLAG_LSS_MAX_DENSITY]) {
     linearSparseSolverMaxDensity = atof(omc_flagValue[FLAG_LSS_MAX_DENSITY]);
@@ -841,6 +1003,14 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
     linearSparseSolverMinSize = atoi(omc_flagValue[FLAG_LSS_MIN_SIZE]);
     infoStreamPrint(LOG_STDOUT, 0, "Maximum system size for using linear sparse solver changed to %d", linearSparseSolverMinSize);
   }
+  if(omc_flag[FLAG_NLS_MAX_DENSITY]) {
+    nonlinearSparseSolverMaxDensity = atof(omc_flagValue[FLAG_NLS_MAX_DENSITY]);
+    infoStreamPrint(LOG_STDOUT, 0, "Maximum density for using non-linear sparse solver changed to %f", nonlinearSparseSolverMaxDensity);
+  }
+  if(omc_flag[FLAG_NLS_MIN_SIZE]) {
+    nonlinearSparseSolverMinSize = atoi(omc_flagValue[FLAG_NLS_MIN_SIZE]);
+    infoStreamPrint(LOG_STDOUT, 0, "Maximum system size for using non-linear sparse solver changed to %d", nonlinearSparseSolverMinSize);
+  }
   if(omc_flag[FLAG_NEWTON_XTOL]) {
     newtonXTol = atof(omc_flagValue[FLAG_NEWTON_XTOL]);
     infoStreamPrint(LOG_STDOUT, 0, "Tolerance for updating solution vector in Newton solver changed to %g", newtonXTol);
@@ -851,10 +1021,56 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
     infoStreamPrint(LOG_STDOUT, 0, "Tolerance for accepting accuracy in Newton solver changed to %g", newtonFTol);
   }
 
+  if(omc_flag[FLAG_NEWTON_MAX_STEP_FACTOR]) {
+    maxStepFactor = atof(omc_flagValue[FLAG_NEWTON_MAX_STEP_FACTOR]);
+    infoStreamPrint(LOG_STDOUT, 0, "Maximum step size factor for a Newton step changed to %g", newtonFTol);
+  }
+
+  if(omc_flag[FLAG_STEADY_STATE_TOL]) {
+    steadyStateTol = atof(omc_flagValue[FLAG_STEADY_STATE_TOL]);
+    infoStreamPrint(LOG_STDOUT, 0, "Tolerance for steady state detection changed to %g", steadyStateTol);
+  }
+
+  if(omc_flag[FLAG_DAE_MODE]) {
+    warningStreamPrint(LOG_STDOUT, 0, "The daeMode flag is *deprecated*, because it is not needed any more.\n"
+      "If a model is compiled in \"DAEmode\" with compiler flag --daeMode, then it simulates automatically in DAE mode.");
+  }
+
   rt_tick(SIM_TIMER_INIT_XML);
   read_input_xml(data->modelData, data->simulationInfo);
+  data->simulationInfo->minStepSize = 4.0 * DBL_EPSILON * fmax(fabs(data->simulationInfo->startTime),fabs(data->simulationInfo->stopTime));
   rt_accumulate(SIM_TIMER_INIT_XML);
 
+  /* Set the maximum number of threads prior to any allocation w.r.t.
+   * linear systems and Jacobians in order to avoid memory leaks.
+   */
+#ifdef USE_PARJAC
+  int num_threads = omc_get_max_threads();
+  if (omc_flag[FLAG_JACOBIAN_THREADS]) {
+    int num_threads_tmp = atoi(omc_flagValue[FLAG_JACOBIAN_THREADS]);
+    infoStreamPrint(LOG_STDOUT, 0,
+         "Number of threads passed via -jacobianThreads: %d",
+         num_threads_tmp);
+    if (0 >= num_threads_tmp) {
+      warningStreamPrint(LOG_STDOUT, 0,
+          "Number of desired OpenMP threads for parallel Jacobian evaluation is <= 0.");
+      warningStreamPrint(LOG_STDOUT, 0, "Use omp_get_max_threads().");
+    } else {
+      num_threads = num_threads_tmp;
+    }
+  }
+  omp_set_num_threads(num_threads);
+
+  infoStreamPrint(LOG_STDOUT, 0,
+      "Number of OpenMP threads for parallel Jacobian evaluation: %d",
+      omc_get_max_threads());
+#else
+  if (omc_flag[FLAG_JACOBIAN_THREADS]) {
+      warningStreamPrint(LOG_STDOUT, 0,
+          "Simulation flag jacobianThreads not available. Make sure you have configured omc with \"--enable-parjac\" and build with a compiler supporting OpenMP.");
+  }
+#endif
+
   /* initialize static data of mixed/linear/non-linear system solvers */
   initializeMixedSystems(data, threadData);
   initializeLinearSystems(data, threadData);
@@ -864,22 +1080,10 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
 
 #ifndef NO_INTERACTIVE_DEPENDENCY
   if(omc_flag[FLAG_PORT]) {
-    std::istringstream stream(omc_flagValue[FLAG_PORT]);
-    int port;
-    stream >> port;
-    sim_communication_port_open = 1;
-    sim_communication_port_open &= sim_communication_port.create();
-    sim_communication_port_open &= sim_communication_port.connect("127.0.0.1", port);
-
     if(0 != strcmp("ia", data->simulationInfo->outputFormat)) {
       communicateStatus("Starting", 0.0, data->simulationInfo->startTime, 0);
     }
   }
-
-  if (isXMLTCP && !sim_communication_port_open) {
-    errorStreamPrint(LOG_STDOUT, 0, "xmltcp log format requires a TCP-port to be passed (and successfully open)");
-    EXIT(1);
-  }
 #endif
   // ppriv - NO_INTERACTIVE_DEPENDENCY - for simpler debugging in Visual Studio
 
@@ -930,26 +1134,39 @@ void communicateMsg(char id, unsigned int size, const char *data)
 
 int _main_SimulationRuntime(int argc, char**argv, DATA *data, threadData_t *threadData)
 {
-  int retVal = -1;
-  MMC_TRY_INTERNAL(globalJumpBuffer)
-    if (initRuntimeAndSimulation(argc, argv, data, threadData)) //initRuntimeAndSimulation returns 1 if an error occurs
-      return 1;
-
-    /* sighandler_t oldhandler = different type on all platforms... */
-#ifdef SIGUSR1
-    SimulationRuntime_printStatus_data = data; /* Global, but at least we get something back; doesn't matter which simulation run */
-    signal(SIGUSR1, SimulationRuntime_printStatus);
+#if defined(__MINGW32__) || defined(_MSC_VER)
+  /* Support for non-ASCII characters
+   * Read the unicode command line arguments and replace the normal arguments with it.
+   */
+  wchar_t** wargv = CommandLineToArgvW(GetCommandLineW(), &argc);
+  for (int i = 0; i < argc; i++) {
+    WIDECHAR_TO_MULTIBYTE_LENGTH(wargv[i], len);
+    WIDECHAR_TO_MULTIBYTE_VAR(wargv[i], buf, len);
+    strcpy(argv[i], buf);
+    MULTIBYTE_OR_WIDECHAR_VAR_FREE(buf);
+  }
 #endif
 
-    retVal = startNonInteractiveSimulation(argc, argv, data, threadData);
+  int retVal = -1;
+  MMC_TRY_INTERNAL(globalJumpBuffer)
+  if (initRuntimeAndSimulation(argc, argv, data, threadData)) //initRuntimeAndSimulation returns 1 if an error occurs
+    return 1;
 
-    freeMixedSystems(data, threadData);        /* free mixed system data */
-    freeLinearSystems(data, threadData);       /* free linear system data */
-    freeNonlinearSystems(data, threadData);    /* free nonlinear system data */
+  /* sighandler_t oldhandler = different type on all platforms... */
+#ifdef SIGUSR1
+  SimulationRuntime_printStatus_data = data; /* Global, but at least we get something back; doesn't matter which simulation run */
+  signal(SIGUSR1, SimulationRuntime_printStatus);
+#endif
 
-    data->callback->callExternalObjectDestructors(data, threadData);
-    deInitializeDataStruc(data);
-    fflush(NULL);
+  retVal = startNonInteractiveSimulation(argc, argv, data, threadData);
+
+  freeMixedSystems(data, threadData);        /* free mixed system data */
+  freeLinearSystems(data, threadData);       /* free linear system data */
+  freeNonlinearSystems(data, threadData);    /* free nonlinear system data */
+
+  data->callback->callExternalObjectDestructors(data, threadData);
+  deInitializeDataStruc(data);
+  fflush(NULL);
   MMC_CATCH_INTERNAL(globalJumpBuffer)
 
 #ifndef NO_INTERACTIVE_DEPENDENCY
