@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -33,15 +33,14 @@
 
 #include "Component.h"
 #include "ComponentFMUME.h"
+#include "ComponentTable.h"
 #include "Flags.h"
 #include "Model.h"
 #include "Types.h"
 #include "ssd/Tags.h"
-#include "cvode/cvode.h"             /* prototypes for CVODE fcts., consts. */
-#include "nvector/nvector_serial.h"  /* serial N_Vector types, fcts., macros */
-#include "cvode/cvode_dense.h"       /* prototype for CVDense */
-#include "sundials/sundials_dense.h" /* definitions DlsMat DENSE_ELEM */
-#include "sundials/sundials_types.h" /* definition of type realtype */
+#include <cvode/cvode.h>                /* prototypes for CVODE fcts., consts. */
+#include <nvector/nvector_serial.h>     /* serial N_Vector types, fcts., macros */
+#include <sunlinsol/sunlinsol_dense.h>  /* Default dense linear solver */
 
 int oms::cvode_rhs(realtype t, N_Vector y, N_Vector ydot, void* user_data)
 {
@@ -65,7 +64,7 @@ int oms::cvode_rhs(realtype t, N_Vector y, N_Vector ydot, void* user_data)
   //std::cout << "[oms::cvode_rhs] y" << std::endl;
   //N_VPrint_Serial(y);
 
-  system->updateInputs(system->outputsGraph);
+  system->updateInputs(system->eventGraph);
 
   // get state derivatives
   for (int j=0, k=0; j < system->fmus.size(); ++j)
@@ -119,9 +118,9 @@ std::string oms::SystemSC::getSolverName() const
       return std::string("euler");
     case oms_solver_sc_cvode:
       return std::string("cvode");
+    default:
+      return std::string("unknown");
   }
-
-  return std::string("unknown");
 }
 
 oms_status_enu_t oms::SystemSC::setSolverMethod(std::string solver)
@@ -138,9 +137,14 @@ oms_status_enu_t oms::SystemSC::setSolverMethod(std::string solver)
 
 oms_status_enu_t oms::SystemSC::exportToSSD_SimulationInformation(pugi::xml_node& node) const
 {
-  pugi::xml_node node_simulation_information = node.append_child(oms::ssd::ssd_simulation_information);
+  pugi::xml_node node_annotations = node.append_child(oms::ssp::Draft20180219::ssd::annotations);
+  pugi::xml_node node_annotation = node_annotations.append_child(oms::ssp::Version1_0::ssc::annotation);
+  node_annotation.append_attribute("type") = oms::ssp::Draft20180219::annotation_type;
 
-  pugi::xml_node node_solver = node_simulation_information.append_child("VariableStepSolver");
+  /* ssd:SimulationInformation should be added as vendor specific annotations from Version 1.0 */
+  pugi::xml_node node_simulation_information = node_annotation.append_child(oms::ssp::Version1_0::simulation_information);
+
+  pugi::xml_node node_solver = node_simulation_information.append_child(oms::ssp::Version1_0::VariableStepSolver);
   node_solver.append_attribute("description") = getSolverName().c_str();
   node_solver.append_attribute("absoluteTolerance") = std::to_string(absoluteTolerance).c_str();
   node_solver.append_attribute("relativeTolerance") = std::to_string(relativeTolerance).c_str();
@@ -151,16 +155,31 @@ oms_status_enu_t oms::SystemSC::exportToSSD_SimulationInformation(pugi::xml_node
   return oms_status_ok;
 }
 
-oms_status_enu_t oms::SystemSC::importFromSSD_SimulationInformation(const pugi::xml_node& node)
+oms_status_enu_t oms::SystemSC::importFromSSD_SimulationInformation(const pugi::xml_node& node, const std::string& sspVersion)
 {
-  std::string solverName = node.child("VariableStepSolver").attribute("description").as_string();
+  std::string solverName = "";
+  const char* VariableStepSolver = "";
+
+  pugi::xml_node variableStepSolver = node.child(oms::ssp::Version1_0::VariableStepSolver);
+  if (variableStepSolver)
+  {
+    solverName = variableStepSolver.attribute("description").as_string();
+    VariableStepSolver = oms::ssp::Version1_0::VariableStepSolver;
+  }
+  else
+  {
+    solverName = node.child("VariableStepSolver").attribute("description").as_string();
+    VariableStepSolver = "VariableStepSolver";
+  }
+
   if (oms_status_ok != setSolverMethod(solverName))
     return oms_status_error;
-  absoluteTolerance = node.child("VariableStepSolver").attribute("absoluteTolerance").as_double();
-  relativeTolerance = node.child("VariableStepSolver").attribute("relativeTolerance").as_double();
-  minimumStepSize = node.child("VariableStepSolver").attribute("minimumStepSize").as_double();
-  maximumStepSize = node.child("VariableStepSolver").attribute("maximumStepSize").as_double();
-  initialStepSize = node.child("VariableStepSolver").attribute("initialStepSize").as_double();
+
+  absoluteTolerance = node.child(VariableStepSolver).attribute("absoluteTolerance").as_double();
+  relativeTolerance = node.child(VariableStepSolver).attribute("relativeTolerance").as_double();
+  minimumStepSize = node.child(VariableStepSolver).attribute("minimumStepSize").as_double();
+  maximumStepSize = node.child(VariableStepSolver).attribute("maximumStepSize").as_double();
+  initialStepSize = node.child(VariableStepSolver).attribute("initialStepSize").as_double();
   return oms_status_ok;
 }
 
@@ -168,7 +187,7 @@ oms_status_enu_t oms::SystemSC::instantiate()
 {
   time = getModel()->getStartTime();
 
-  // there shouldn't be any substem
+  // there shouldn't be any subsystem
   for (const auto& subsystem : getSubSystems())
     if (oms_status_ok != subsystem.second->instantiate())
       return oms_status_error;
@@ -221,7 +240,7 @@ oms_status_enu_t oms::SystemSC::initialize()
   if (oms_status_ok != updateDependencyGraphs())
     return oms_status_error;
 
-  if (oms_status_ok != updateInputs(initialUnknownsGraph))
+  if (oms_status_ok != updateInputs(initializationGraph))
     return oms_status_error;
 
   for (const auto& subsystem : getSubSystems())
@@ -274,7 +293,7 @@ oms_status_enu_t oms::SystemSC::initialize()
 
     // Call CVodeCreate to create the solver memory and specify the
     // Backward Differentiation Formula and the use of a Newton iteration
-    solverData.cvode.mem = CVodeCreate(CV_BDF, CV_NEWTON);
+    solverData.cvode.mem = CVodeCreate(CV_BDF);
     if (!solverData.cvode.mem) logError("SUNDIALS_ERROR: CVodeCreate() failed - returned NULL pointer");
 
     int flag = CVodeSetUserData(solverData.cvode.mem, (void*)this);
@@ -291,8 +310,18 @@ oms_status_enu_t oms::SystemSC::initialize()
     flag = CVodeSVtolerances(solverData.cvode.mem, relativeTolerance, solverData.cvode.abstol);
     if (flag < 0) logError("SUNDIALS_ERROR: CVodeSVtolerances() failed with flag = " + std::to_string(flag));
 
-    // Call CVDense to specify the CVDENSE dense linear solver */
-    flag = CVDense(solverData.cvode.mem, static_cast<long>(n_states));
+    // Call N_VNew_Serial and SUNDenseMatrix to generate dense vector abd natrix for lin. solver module
+    solverData.cvode.liny = N_VNew_Serial(n_states);
+    if (solverData.cvode.liny == NULL) logError("SUNDIALS_ERROR: N_VNew_Serial() failed");
+    solverData.cvode.J = SUNDenseMatrix(n_states, n_states);
+    if (solverData.cvode.J == NULL) logError("SUNDIALS_ERROR: N_VNew_Serial() failed");
+
+    // Call SUNLinSol_Dense to creat linear solver object
+    solverData.cvode.linSol = SUNLinSol_Dense(solverData.cvode.liny, solverData.cvode.J);
+    if (solverData.cvode.linSol == NULL) logError("SUNDIALS_ERROR: SUNLinSol_Dense() failed");
+
+    // Call CVodeSetLinearSolver to set the dense linear solver */
+    flag = CVodeSetLinearSolver(solverData.cvode.mem, solverData.cvode.linSol, solverData.cvode.J);
     if (flag < 0) logError("SUNDIALS_ERROR: CVDense() failed with flag = " + std::to_string(flag));
 
     const fmi2_real_t stopTime = 1.0;
@@ -308,7 +337,7 @@ oms_status_enu_t oms::SystemSC::initialize()
     if (flag < 0) logError("SUNDIALS_ERROR: CVodeSetMaxOrd() failed with flag = " + std::to_string(flag));
     flag = CVodeSetMaxConvFails(solverData.cvode.mem, 100);     // MAXIMUM NUMBER OF NONLINEAR CONVERGENCE FAILURES
     if (flag < 0) logError("SUNDIALS_ERROR: CVodeSetMaxConvFails() failed with flag = " + std::to_string(flag));
-    flag = CVodeSetStabLimDet(solverData.cvode.mem, TRUE);      // STABILITY DETECTION
+    flag = CVodeSetStabLimDet(solverData.cvode.mem, true);      // STABILITY DETECTION
     if (flag < 0) logError("SUNDIALS_ERROR: CVodeSetStabLimDet() failed with flag = " + std::to_string(flag));
     flag = CVodeSetMinStep(solverData.cvode.mem, 1e-12);        // MINIMUM STEPSIZE
     if (flag < 0) logError("SUNDIALS_ERROR: CVodeSetMinStep() failed with flag = " + std::to_string(flag));
@@ -356,6 +385,9 @@ oms_status_enu_t oms::SystemSC::terminate()
     msg += "NumNonlinSolvIters = " + std::to_string(nni) + " NumNonlinSolvConvFails = " + std::to_string(ncfn) + " NumErrTestFails = " + std::to_string(netf);
     logInfo(msg);
 
+    SUNMatDestroy(solverData.cvode.J);
+    N_VDestroy(solverData.cvode.liny);
+    SUNLinSolFree(solverData.cvode.linSol);
     N_VDestroy_Serial(solverData.cvode.y);
     N_VDestroy_Serial(solverData.cvode.abstol);
     CVodeFree(&(solverData.cvode.mem));
@@ -433,9 +465,6 @@ oms_status_enu_t oms::SystemSC::stepUntil(double stopTime, void (*cb)(const char
 {
   CallClock callClock(clock);
 
-  if (0 == fmus.size())
-    return oms_status_ok;
-
   ComRef modelName = this->getModel()->getCref();
   fmi2_status_t fmistatus;
   oms_status_enu_t status;
@@ -483,6 +512,12 @@ oms_status_enu_t oms::SystemSC::stepUntil(double stopTime, void (*cb)(const char
       // handle events
       if (callEventUpdate[i] || zero_crossing_event || (fmus[i]->getEventInfo()->nextEventTimeDefined && time == fmus[i]->getEventInfo()->nextEventTime))
       {
+        logDebug("Event detected in FMU \"" + std::string(fmus[i]->getFullCref()) + "\" at time=" + std::to_string(time));
+
+        // emit the left limit of the event (if it hasn't already been emitted)
+        if (Flags::EmitEvents() && isTopLevelSystem())
+          getModel()->emit(time, false);
+
         fmistatus = fmi2_import_enter_event_mode(fmus[i]->getFMU());
         if (fmi2_status_ok != fmistatus) logError_FMUCall("fmi2_import_enter_event_mode", fmus[i]);
 
@@ -510,6 +545,15 @@ oms_status_enu_t oms::SystemSC::stepUntil(double stopTime, void (*cb)(const char
           int flag = CVodeReInit(solverData.cvode.mem, time, solverData.cvode.y);
           if (flag < 0) logError("SUNDIALS_ERROR: CVodeReInit() failed with flag = " + std::to_string(flag));
         }
+
+        // emit the right limit of the event
+        updateInputs(eventGraph);
+        if (Flags::EmitEvents() && isTopLevelSystem())
+          getModel()->emit(time, true);
+
+        // restart event iteration from the beginning
+        i=-1;
+        continue;
       }
 
       // calculate next time step
@@ -579,10 +623,10 @@ oms_status_enu_t oms::SystemSC::stepUntil(double stopTime, void (*cb)(const char
     for (int i=0; i < fmus.size(); ++i)
     {
       fmistatus = fmi2_import_completed_integrator_step(fmus[i]->getFMU(), fmi2_true, &callEventUpdate[i], &terminateSimulation[i]);
-      if (fmi2_status_ok != fmistatus) logError_FMUCall("fmi2_import_completed_integrator_step", fmus[i]);
+      if (fmi2_status_ok != fmistatus) return logError_FMUCall("fmi2_import_completed_integrator_step", fmus[i]);
     }
 
-    updateInputs(outputsGraph);
+    updateInputs(simulationGraph); //pass the continuousTimeMode dependency graph which involves only connections of type Real
     if (isTopLevelSystem())
       getModel()->emit(time);
 
@@ -612,6 +656,26 @@ oms_status_enu_t oms::SystemSC::updateInputs(DirectedGraph& graph)
 {
   CallClock callClock(clock);
 
+  if (getModel()->validState(oms_modelState_simulation))
+  {
+    // update time
+    for (const auto& component : getComponents())
+    {
+      switch (component.second->getType())
+      {
+        case oms_component_fmu:
+          if (fmi2_status_ok != fmi2_import_set_time(dynamic_cast<ComponentFMUME*>(component.second)->getFMU(), time))
+            logError_FMUCall("fmi2_import_set_time", dynamic_cast<ComponentFMUME*>(component.second));
+          break;
+        case oms_component_table:
+          dynamic_cast<ComponentTable*>(component.second)->stepUntil(time);
+          break;
+        default:
+          break;
+      }
+    }
+  }
+
   // input := output
   const std::vector< std::vector< std::pair<int, int> > >& sortedConnections = graph.getSortedConnections();
   for(int i=0; i<sortedConnections.size(); i++)
@@ -627,7 +691,7 @@ oms_status_enu_t oms::SystemSC::updateInputs(DirectedGraph& graph)
         if (oms_status_ok != getReal(graph.getNodes()[output].getName(), value)) return oms_status_error;
         if (oms_status_ok != setReal(graph.getNodes()[input].getName(), value)) return oms_status_error;
       }
-      else if (graph.getNodes()[input].getType() == oms_signal_type_integer)
+      else if (graph.getNodes()[input].getType() == oms_signal_type_integer || graph.getNodes()[input].getType() == oms_signal_type_enum)
       {
         int value = 0.0;
         if (oms_status_ok != getInteger(graph.getNodes()[output].getName(), value)) return oms_status_error;
@@ -655,7 +719,7 @@ oms_status_enu_t oms::SystemSC::solveAlgLoop(DirectedGraph& graph, const std::ve
   CallClock callClock(clock);
 
   const int size = SCC.size();
-  const int maxIterations = 10;
+  const int maxIterations = Flags::MaxLoopIteration();
   double maxRes;
   double *res = new double[size]();
 
